/*
 * osdlist.cpp
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include <KIconLoader>

#include "osd.h"
#include "videowidget.h"



Osd::Osd( VideoWidget *parent, GLFunc *glf )
{
	vw = parent;
	osdConfig = new OsdConfig();
	osdConfig->func = glf;

	osdConfig->setButtonHighGrad0Color( QColor(128,0,0,200) );
	osdConfig->setButtonHighGrad1Color( QColor(128,100,100,200) );
	osdConfig->setButtonGrad0Color( QColor(128,128,128,180) );
	osdConfig->setButtonGrad1Color( QColor(0,0,0,180) );
	osdConfig->setForegroundColor( QColor(200,200,200) );
	osdConfig->setForegroundHighColor( Qt::white );
	//osdConfig->setBackgroundColor( QColor( 55,0,0,150) );
	osdConfig->setBackgroundColor( QColor(0,0,0,150) );
	osdConfig->setListItemHighGrad0Color( QColor(128,0,0,200) );
	osdConfig->setListItemHighGrad1Color( QColor(128,100,100,200) );
	osdConfig->setListItemGrad0Color( QColor(0,0,0,200) );
	osdConfig->setListItemGrad1Color( QColor(0,0,0,100) );
	osdConfig->setScrollBarGrad0Color( QColor(128,0,0,180) );
	osdConfig->setScrollBarGrad1Color( QColor(128,100,100,180) );
	osdConfig->setScrollBarBackgroundColor( QColor( 58,50,150,100) );
	osdConfig->setNormalFont( QFont("Sans", 12 ) );

	menu = new OsdMenu( osdConfig );
	menu->setGeometry( QRect(0,0,500,30) );
	menu->setAnimDuration( 200 );
	menu->setShowVector( QPoint(0,1) );
	menu->setHideVector( QPoint(0,-1) );
	menu->setAnimType( OsdAnim::LinearVertical );
	menu->setHidden( false );
	connect( menu, SIGNAL(wantRepaint()), vw, SLOT(repaintRequest()) );
	connect( menu, SIGNAL(startRepaint()), vw, SLOT(startRepaint()) );
	connect( menu, SIGNAL(stopRepaint()), vw, SLOT(stopRepaint()) );
	OsdButton *btn;
	menu->appendButton( btn=new OsdButton( osdConfig, QString("Take thumb") ) );
	connect( btn, SIGNAL(clicked()), this, SLOT(shot()) );
	menu->appendButton( btn=new OsdButton( osdConfig, QString("Toggle deinterlacer") ) );
	connect( btn, SIGNAL(clicked()), vw, SLOT(toggleDeinterlace()) );

	browser = new FileBrowser( osdConfig, vw );
	browser->setGeometry( QRect(20, 50, 250, 600) );
	browser->setAnimDuration( 300 );
	browser->setShowVector( QPoint(1,0) );
	browser->setHideVector( QPoint(-1,0) );
	browser->setAnimType( OsdAnim::LinearHorizontal );
	browser->setHidden( false );
	connect( browser, SIGNAL(wantRepaint()), vw, SLOT(repaintRequest()) );
	connect( browser, SIGNAL(startRepaint()), vw, SLOT(startRepaint()) );
	connect( browser, SIGNAL(stopRepaint()), vw, SLOT(stopRepaint()) );

	control = new OsdControlBar( osdConfig  );
	control->setGeometry( QRect(0,0,500,30) );
	control->setHidden( false );
	connect( control, SIGNAL(wantRepaint()), vw, SLOT(repaintRequest()) );
	connect( control, SIGNAL(startRepaint()), vw, SLOT(startRepaint()) );
	connect( control, SIGNAL(stopRepaint()), vw, SLOT(stopRepaint()) );

	control->appendButton( btn = new OsdButton( osdConfig, "" ) );
	btn->setIcon( KIconLoader::global()->iconPath("media-skip-backward", KIconLoader::Desktop) );
	control->appendButton( btn = new OsdButton( osdConfig, "" ) );
	btn->setIcon( KIconLoader::global()->iconPath("media-playback-start", KIconLoader::Desktop) );
	connect( btn, SIGNAL(clicked()), vw, SLOT(play()) );
	control->appendButton( btn = new OsdButton( osdConfig, "" ) );
	btn->setIcon( KIconLoader::global()->iconPath("media-playback-stop", KIconLoader::Desktop) );
	connect( btn, SIGNAL(clicked()), vw, SLOT(stop()) );
	control->appendButton( btn = new OsdButton( osdConfig, "" ) );
	btn->setIcon( KIconLoader::global()->iconPath("media-skip-forward", KIconLoader::Desktop) );
}



void Osd::shot()
{
	QString s = vw->shot();
	if ( !s.isEmpty() ) {
		browser->updateThumb( s );
	}
}



void Osd::mouseMove( QMouseEvent *e )
{
	QPoint p(e->x(), e->y() );
	menu->mouseMove( p );
	browser->mouseMove( p );
	control->mouseMove( p );
}



bool Osd::mouseClicked( QMouseEvent *e )
{
	if ( e->button()==Qt::RightButton ) {
		//menu->toggle();
		//control->toggle();
		browser->toggle();
		return true;
	}
	if ( browser->mouseClicked( e ) )
		return true;
	if ( menu->mouseClicked( e ) )
		return true;
	if ( control->mouseClicked( e ) )
		return true;
	return false;
}



void Osd::mouseReleased( QMouseEvent *e )
{
	browser->mouseReleased( e );
	control->mouseReleased( e );
}



void Osd::wheelEvent( QWheelEvent *e )
{
	browser->wheelEvent( e );
	control->wheelEvent( e );
}



void Osd::paint()
{
	menu->paint();
	control->paint();
	browser->paint();
}


void Osd::resize( int w, int h )
{
	//int fs = (w+h)*14/(1024+768);
	//osdConfig->setNormalFont( QFont("Sans", fs ) );
	menu->resize( w, h );
	control->resize( w, h );
	browser->resize( w-40, h-100 );
}
