/*
 * osdanim.cpp
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include <KDebug>

#include "osdanim.h"



OsdAnim::OsdAnim( OsdConfig *conf ) : OsdObject( conf )
{
	moving = false;
	hidden = true;
	showVector = QPoint( 0,0 );
	hideVector = QPoint( 0,0 );
	currentVector = &showVector;
	duration = 100;
	timeOffset = 0;
}



OsdAnim::~OsdAnim()
{
}



void OsdAnim::toggle()
{
	if ( moving )
		hidden = !hidden;
	if ( isHidden() )
		show();
	else
		hide();
}



void OsdAnim::show()
{
	currentVector = &showVector;
	kDebug() << "show";
	if ( !moving ) {
		moving = true;
		startTime.start();
		emit startRepaint();
	}
	else {
		timeOffset += duration-startTime.elapsed();
		startTime.start();
	}
}



void OsdAnim::hide()
{
	currentVector = &hideVector;
	kDebug() << "hide";
	if ( !moving ) {
		moving = true;
		startTime.start();
		emit startRepaint();
	}
	else {
		timeOffset += duration-startTime.elapsed();
		startTime.start();
	}
}



void OsdAnim::setTransform()
{
	glPushMatrix();
	if ( moving ) {
		switch ( type ) {
		case None:
			break;
		case LinearHorizontal: {
			qreal off = ((startTime.elapsed()+timeOffset)*rect.width())/duration;
			qreal tmp = off;
			if ( currentVector->x()<0 )
				off = -off;
			else
				off -=rect.width();
			if ( tmp>rect.width() ) {
				hidden = !hidden;
				moving = false;
				timeOffset = 0;
				emit stopRepaint();
				if ( currentVector->x()>0 )
					return;
			}
			glTranslatef( off, 0, 0 );
			break;
		}
		case LinearVertical: {
			qreal off = (startTime.elapsed()*rect.height())/duration;
			qreal tmp = off;
			if ( currentVector->y()<0 )
				off = -off;
			else
				off -=rect.height();
			if ( tmp>rect.height() ) {
				hidden = !hidden;
				moving = false;
				timeOffset = 0;
				emit stopRepaint();
				if ( currentVector->y()>0 )
					return;
			}
			glTranslatef( 0, off, 0 );
			break;
		}
		}
	}
}
